#include "funcs.h"

#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64

#define UP 7
#define DOWN 9
#define LEFT 10
#define RIGHT 8
#define BTN1 4
#define BTN2 5

#define beginX 4
#define beginY 13
const int W = 14;
const int H = 6;

void intro(Adafruit_SSD1306& display) {
	display.clearDisplay();
	display.setTextSize(1);
	display.setTextColor(SSD1306_WHITE);
	display.setCursor(30, 0);
	display.cp437(true);
	display.write("Game of life");
	display.display();
	delay(500);
	display.setCursor(10, 20);
	display.write("Press 1 to enter...");
	display.display();
	display.setCursor(10, 40);
	display.write("Press RED to reset");
	display.display();
	display.drawLine(0, 8, 126, 8, SSD1306_WHITE);
	display.display();
	while (digitalRead(BTN1) == HIGH) {
	}
	delay(500);
}

void buttonRead(bool* moved, bool* sim, int* cursorX, int* cursorY, bool** cell) {
	if (*sim == false) {
		if (digitalRead(UP) == LOW) {
			*cursorY -= 9;
			if (*cursorY < 9) *cursorY = SCREEN_HEIGHT - 6;
			*moved = true;
		}
		if (digitalRead(DOWN) == LOW) {
			*cursorY += 9;
			if (*cursorY >= SCREEN_HEIGHT) *cursorY = 13;
			*moved = true;
		}
		if (digitalRead(LEFT) == LOW) {
			*cursorX -= 9;
			if (*cursorX < 5) *cursorX = SCREEN_WIDTH - 6;
			*moved = true;
		}
		if (digitalRead(RIGHT) == LOW) {
			*cursorX += 9;
			if (*cursorX >= SCREEN_WIDTH - 5) *cursorX = 5;
			*moved = true;
		}
		if (digitalRead(BTN1) == LOW) {
			for (int i = 0; i < W; i++) {
				for (int j = 0; j < H; j++) {
					if (*cursorX >= 9 * i && *cursorX <= 9 * i + 8 && *cursorY >= 9 * j + 13 && *cursorY <= 9 * j + 13 + 8) {
						if (cell[i][j] == true) cell[i][j] = false;
						else cell[i][j] = true;
					}
				}
			}
		}
	}
	if (digitalRead(BTN2) == LOW && *sim == false) {
		*sim = true;
		delay(200);
	} else *sim = false;
}

void drawGrid(Adafruit_SSD1306& display) {
	for (int i = 8; i <= SCREEN_HEIGHT; i += 9) {
		display.drawLine(0, i, SCREEN_WIDTH - 2, i, SSD1306_WHITE);
	}
	for (int j = 0; j <= SCREEN_WIDTH; j += 9) {
		display.drawLine(j, 8, j, SCREEN_HEIGHT - 2, SSD1306_WHITE);
	}
	display.setTextSize(1);
	display.setTextColor(SSD1306_WHITE);
	display.setCursor(30, 0);
	display.cp437(true);
	display.write("Game of life");
}

void fill_cell(Adafruit_SSD1306& display, int m, int n, bool** cell) {
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < m; j++) {
			if (cell[j][i] == true) {
				//display.fillCircle(9 * j + beginX, 9 * i + beginY, 3, SSD1306_WHITE);
				display.fillRect(9 * j + 1 + 1, 9 * i + 9 + 1, 6, 6, SSD1306_WHITE);
			}
		}
	}
}

void draw_cell(Adafruit_SSD1306& display, int i, int j) {
	//display.fillCircle(9 * j + beginX, 9 * i + beginY, 3, SSD1306_WHITE);
	display.fillRect(9 * j + 1 + 1, 9 * i + 9 + 1, 6, 6, SSD1306_WHITE);
}

void check_neighbours(int m, int n, bool** cell1, bool** tab) {
	//-- "tab" - pomocnicza tablicza tablica będąca kopią tablicy komórek --
	for (int i = 0; i < m; i++) {
		for (int j = 0; j < n; j++) {
			tab[i][j] = cell1[i][j];
		}
	}

	int count;

	for (int i = 0; i < m; i++) {
		for (int j = 0; j < n; j++) {
			count = 0;

			// sprawdzanie sąsiadów z "zawijaniem"
			for (int dx = -1; dx <= 1; dx++) {
				for (int dy = -1; dy <= 1; dy++) {
					if (dx == 0 && dy == 0) continue;  // pomija samą siebie

					int ni = (i + dx + m) % m;
					int nj = (j + dy + n) % n;

					count += (int)cell1[ni][nj];
				}
			}

			cell_check(i, j, count, cell1, tab);
		}
	}

	for (int i = 0; i < m; i++) {
		for (int j = 0; j < n; j++) {
			cell1[i][j] = tab[i][j];
		}
	}
}

void cell_check(int i, int j, int count, bool** cell1, bool** tab) {
	if (cell1[i][j] == 0)  //-- komórka jest martwa --
	{
		switch (count) {
			case (3):  //-- 3 żywe = komórka staje się żywa --
				tab[i][j] = 1;
				break;
		}
	} else switch (count)  //-- komórka jest żywa --
	{
			case (2):  //-- 2 lub 3 żywe = pozostaje żywa --
				tab[i][j] = 1;
				break;
			case (3):
				tab[i][j] = 1;
				break;
			default:  //-- więcej niż 3 żywe (>=4) = umiera --
				tab[i][j] = 0;
				break;
		}
}

void fill_tab(bool value, int m, int n, bool** cell) {
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < m; j++) {
			cell[j][i] = value;
		}
	}
}

void clone_tab(int m, int n, bool** tab1, bool** tab2) {
	for (int i = 0; i < m; i++) {
		for (int j = 0; j < n; j++) {
			tab2[i][j] = tab1[i][j];
		}
	}
}