#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128  // OLED display width, in pixels
#define SCREEN_HEIGHT 64  // OLED display height, in pixels

#define OLED_RESET -1        // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C  ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

// 14 x 6 - plansza
#define UP 7
#define DOWN 9
#define LEFT 10
#define RIGHT 8
#define BTN1 4
#define BTN2 5

#include "funcs.h"

int cursorX = 5;
int cursorY = 9 + 4;
bool cursorVisible = true;
unsigned long previousBlinkMillis = 0;
const unsigned long blinkInterval = 250;

//  zmienne pomocnicze
const int W = 14;
const int H = 6;
bool** cell = NULL;
bool** check = NULL;
bool RunSim;

void setup() {
  Serial.begin(9600);
  pinMode(UP, INPUT_PULLUP);
  pinMode(DOWN, INPUT_PULLUP);
  pinMode(LEFT, INPUT_PULLUP);
  pinMode(RIGHT, INPUT_PULLUP);
  pinMode(BTN1, INPUT_PULLUP);
  pinMode(BTN2, INPUT_PULLUP);

  cell = (bool**)malloc(W * sizeof(bool*));
  for (int i = 0; i < W; i++) cell[i] = (bool*)malloc(H * sizeof(bool));

  check = (bool**)malloc(W * sizeof(bool*));
  for (int i = 0; i < W; i++) check[i] = (bool*)malloc(H * sizeof(bool));

  fill_tab(false, W, H, cell);
  fill_tab(true, W, H, check);

  RunSim = false;

  if (!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;)
      ;
  }

  display.setRotation(2);

  display.clearDisplay();
  intro(display);
}

void loop() {
  bool moved = false;

  buttonRead(&moved, &RunSim, &cursorX, &cursorY, cell);

  if (moved) {
    cursorVisible = true;
    previousBlinkMillis = millis();
  }

  while (RunSim) {
    check_neighbours(W, H, cell, check);
    display.clearDisplay();
    drawGrid(display);
    fill_cell(display, W, H, cell);
    display.display();
    delay(250);
    if(digitalRead(BTN2) == LOW) RunSim = false;
  }

  // Handle blinking
  unsigned long currentMillis = millis();
  if (currentMillis - previousBlinkMillis >= blinkInterval) {
    previousBlinkMillis = currentMillis;
    if (RunSim == false) {
      cursorVisible = !cursorVisible;
    } else cursorVisible = false;
  }

  // Draw display
  display.clearDisplay();
  drawGrid(display);
  fill_cell(display, W, H, cell);

  if (cursorVisible) {
    display.fillCircle(cursorX - 1, cursorY - 1, 1, SSD1306_WHITE);
  } else {
    display.fillCircle(cursorX - 1, cursorY - 1, 1, SSD1306_BLACK);
  }
  display.display();

  delay(100);
}
